@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2022 The LumiSpy developers
+# Copyright 2019-2023 The LumiSpy developers
 #
 # This file is part of LumiSpy.
 #
@@ -79,7 +79,6 @@ class CLSpectrum(LumiSpectrum):
         interactive=False,
         **kwargs
     ):
-
         if luminescence_roi is not None and signal_mask is not None:
             raise AttributeError(
                 "Only either `luminescence_roi` or the `signal_mask` can be an input."
@@ -112,7 +111,7 @@ class CLSpectrum(LumiSpectrum):
             )
 
         if inplace:
-            return
+            return None
         else:
             return signal
 
