@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2022 The LumiSpy developers
+# Copyright 2019-2023 The LumiSpy developers
 #
 # This file is part of LumiSpy.
 #
@@ -24,10 +24,12 @@ from warnings import warn
 from hyperspy.signals import Signal1D
 from hyperspy._signals.lazy import LazySignal
 from hyperspy.axes import DataAxis
+from traits.api import Undefined
 
 from lumispy.signals.common_luminescence import CommonLumi
 from lumispy import nm2invcm, to_array, savetxt
 from lumispy.utils.axes import GRATING_EQUATION_DOCSTRING_PARAMETERS
+from lumispy.utils.signals import com
 from lumispy.utils import (
     axis2eV,
     data2eV,
@@ -93,6 +95,81 @@ class LumiSpectrum(Signal1D, CommonLumi):
                 UserWarning,
             )
 
+    def _convert_data(self, newaxis, factor, inplace, jacobian, data2, var2):
+        """Utility function to perform the data and variance conversion for
+        signal unit transformations.
+        """
+        # convert data
+        if jacobian:
+            s2data = np.copy(
+                data2(
+                    self.isig[::-1].data,
+                    factor,
+                    newaxis.axis,
+                    self.axes_manager.signal_axes[0],
+                )
+            )
+        else:
+            s2data = np.copy(self.isig[::-1].data)
+
+        # inplace conversion
+        if inplace:
+            s2 = self
+            s2.data = s2data
+        # create new signal object with correct data, axes, metadata
+        else:
+            s2 = self._deepcopy_with_new_data(
+                s2data,
+                copy_variance=(not jacobian),
+                copy_learning_results=False,
+            )
+        # convert axis
+        oldaxis = self.axes_manager.signal_axes[0]
+        s2.axes_manager.set_axis(
+            newaxis,
+            self.axes_manager.signal_axes[0].index_in_axes_manager,
+        )
+        # convert variance
+        if self.metadata.has_item("Signal.Noise_properties.variance"):
+            var = self.get_noise_variance()
+            if jacobian:
+                # if variance is a numeric value, cast into signal object
+                if isinstance(var, (float, int)):
+                    var = self._deepcopy_with_new_data(
+                        np.ones(self.data.shape) * var,
+                        copy_variance=False,
+                        copy_learning_results=False,
+                    )
+                s2var = s2._deepcopy_with_new_data(
+                    var2(
+                        var.isig[::-1].data,
+                        factor,
+                        newaxis.axis,
+                        oldaxis,
+                    ),
+                    copy_variance=False,
+                    copy_learning_results=False,
+                )
+                s2.set_noise_variance(s2var)
+                s2._reset_variance_linear_model()
+            else:
+                # variance left unchanged, if it is a number and Jacobian not performed
+                if isinstance(var, (float, int)):
+                    if not inplace:
+                        s2.set_noise_variance(self.get_noise_variance())
+                else:
+                    s2.set_noise_variance(
+                        s2._deepcopy_with_new_data(
+                            var.isig[::-1].data,
+                            copy_variance=False,
+                            copy_learning_results=False,
+                        )
+                    )
+        if not inplace:
+            return s2
+        else:
+            return None
+
     def to_eV(self, inplace=True, jacobian=True):
         """Converts signal axis of 1D signal to non-linear energy axis (eV)
         using wavelength dependent refractive index of air. Assumes wavelength
@@ -128,120 +205,7 @@ class LumiSpectrum(Signal1D, CommonLumi):
 
         evaxis, factor = axis2eV(self.axes_manager.signal_axes[0])
 
-        # in place conversion
-        if inplace:
-            # convert data
-            if jacobian:
-                self.data = data2eV(
-                    self.isig[::-1].data,
-                    factor,
-                    self.axes_manager.signal_axes[0],
-                    evaxis.axis,
-                )
-            else:
-                self.data = self.isig[::-1].data
-            # convert axis
-            oldaxis = self.axes_manager.signal_axes[0]
-            self.axes_manager.set_axis(
-                evaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-            )
-            # convert variance
-            if jacobian:
-                if self.metadata.has_item("Signal.Noise_properties.variance"):
-                    var = self.get_noise_variance()
-                    # if variance is a numeric value, cast into signal object
-                    if isinstance(var, (float, int)):
-                        self.set_noise_variance(
-                            self._deepcopy_with_new_data(
-                                np.ones(self.data.shape) * var,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-                        var = self.get_noise_variance()
-                    svar = self._deepcopy_with_new_data(
-                        var2eV(
-                            var.isig[::-1].data,
-                            factor,
-                            oldaxis,
-                            evaxis.axis,
-                        ),
-                        copy_variance=False,
-                        copy_learning_results=False,
-                    )
-                    self.set_noise_variance(svar)
-                    self._reset_variance_linear_model()
-            else:
-                if self.metadata.has_item("Signal.Noise_properties.variance"):
-                    var = self.get_noise_variance()
-                    # variance left unchanged, if it is a number and Jacobian not performed
-                    if not isinstance(var, (float, int)):
-                        self.set_noise_variance(
-                            self._deepcopy_with_new_data(
-                                var.isig[::-1].data,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-
-        # create and return new signal
-        else:
-            # create new data array
-            if jacobian:
-                s2data = np.copy(
-                    data2eV(
-                        self.isig[::-1].data,
-                        factor,
-                        self.axes_manager.signal_axes[0],
-                        evaxis.axis,
-                    )
-                )
-            else:
-                s2data = np.copy(self.isig[::-1].data)
-
-            # create new signal object with correct data, axes, metadata
-            s2 = self._deepcopy_with_new_data(
-                s2data, copy_variance=(not jacobian), copy_learning_results=False
-            )
-            s2.axes_manager.set_axis(
-                evaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-            )
-            # convert variance
-            if self.metadata.has_item("Signal.Noise_properties.variance"):
-                var = self.get_noise_variance()
-                if jacobian:
-                    # if variance is a numeric value, cast into signal object
-                    if isinstance(var, (float, int)):
-                        var = self._deepcopy_with_new_data(
-                            np.ones(self.data.shape) * var,
-                            copy_variance=False,
-                            copy_learning_results=False,
-                        )
-                    # convert
-                    s2var = s2._deepcopy_with_new_data(
-                        var2eV(
-                            var.isig[::-1].data,
-                            factor,
-                            self.axes_manager.signal_axes[0],
-                            evaxis.axis,
-                        ),
-                        copy_variance=False,
-                        copy_learning_results=False,
-                    )
-                    s2.set_noise_variance(s2var)
-                else:
-                    if isinstance(var, (float, int)):
-                        s2.set_noise_variance(self.get_noise_variance())
-                    else:
-                        s2.set_noise_variance(
-                            s2._deepcopy_with_new_data(
-                                var.isig[::-1].data,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-
-            return s2
+        return self._convert_data(evaxis, factor, inplace, jacobian, data2eV, var2eV)
 
     TO_INVCM_DOCSTRING = """
         The intensity is converted from counts/nm (counts/µm) to counts/cm^-1
@@ -284,114 +248,9 @@ class LumiSpectrum(Signal1D, CommonLumi):
 
         invcmaxis, factor = axis2invcm(self.axes_manager.signal_axes[0])
 
-        # in place conversion
-        if inplace:
-            # convert data
-            if jacobian:
-                self.data = data2invcm(
-                    self.isig[::-1].data,
-                    factor,
-                    invcmaxis.axis,
-                )
-            else:
-                self.data = self.isig[::-1].data
-            # convert axis
-            self.axes_manager.set_axis(
-                invcmaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-            )
-            # convert variance
-            if jacobian:
-                if self.metadata.has_item("Signal.Noise_properties.variance"):
-                    var = self.get_noise_variance()
-                    # if variance is a numeric value, cast into signal object
-                    if isinstance(var, (float, int)):
-                        self.set_noise_variance(
-                            self._deepcopy_with_new_data(
-                                np.ones(self.data.shape) * var,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-                        var = self.get_noise_variance()
-                    # convert
-                    svar = self._deepcopy_with_new_data(
-                        var2invcm(
-                            var.isig[::-1].data,
-                            factor,
-                            invcmaxis.axis,
-                        ),
-                        copy_variance=False,
-                        copy_learning_results=False,
-                    )
-                    self.set_noise_variance(svar)
-                    self._reset_variance_linear_model()
-            else:
-                if self.metadata.has_item("Signal.Noise_properties.variance"):
-                    var = self.get_noise_variance()
-                    # variance left unchanged, if it is a number and Jacobian not performed
-                    if not isinstance(var, (float, int)):
-                        self.set_noise_variance(
-                            self._deepcopy_with_new_data(
-                                var.isig[::-1].data,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-
-        # create and return new signal
-        else:
-            # create new data array
-            if jacobian:
-                s2data = np.copy(
-                    data2invcm(
-                        self.isig[::-1].data,
-                        factor,
-                        invcmaxis.axis,
-                    )
-                )
-            else:
-                s2data = np.copy(self.isig[::-1].data)
-
-            # create new signal object with correct data, axes, metadata
-            s2 = self._deepcopy_with_new_data(
-                s2data, copy_variance=(not jacobian), copy_learning_results=False
-            )
-            s2.axes_manager.set_axis(
-                invcmaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-            )
-            # convert variance
-            if self.metadata.has_item("Signal.Noise_properties.variance"):
-                var = self.get_noise_variance()
-                if jacobian:
-                    # if variance is a numeric value, cast into signal object
-                    if isinstance(var, (float, int)):
-                        var = self._deepcopy_with_new_data(
-                            np.ones(self.data.shape) * var,
-                            copy_variance=False,
-                            copy_learning_results=False,
-                        )
-                    s2var = s2._deepcopy_with_new_data(
-                        var2invcm(
-                            var.isig[::-1].data,
-                            factor,
-                            invcmaxis.axis,
-                        ),
-                        copy_variance=False,
-                        copy_learning_results=False,
-                    )
-                    s2.set_noise_variance(s2var)
-                else:
-                    if isinstance(var, (float, int)):
-                        s2.set_noise_variance(var)
-                    else:
-                        s2.set_noise_variance(
-                            s2._deepcopy_with_new_data(
-                                var.isig[::-1].data,
-                                copy_variance=False,
-                                copy_learning_results=False,
-                            )
-                        )
-            return s2
+        return self._convert_data(
+            invcmaxis, factor, inplace, jacobian, data2invcm, var2invcm
+        )
 
     to_invcm.__doc__ %= (TO_INVCM_DOCSTRING, TO_INVCM_EXAMPLE)
 
@@ -452,43 +311,36 @@ class LumiSpectrum(Signal1D, CommonLumi):
         invcmaxis.name = "Raman Shift"
 
         # replace signal axis after conversion
-        # in place conversion (using absolute scale for Jacobian)
+        # conversion (using absolute scale for Jacobian)
         if inplace:
             self.to_invcm(inplace=inplace, jacobian=jacobian)
-            self.axes_manager.set_axis(
-                invcmaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-            )
-            self.data = self.isig[::-1].data
-            # replace variance axis
-            if self.metadata.has_item(
-                "Signal.Noise_properties.variance"
-            ) and not isinstance(self.get_noise_variance(), (float, int)):
-                self.metadata.Signal.Noise_properties.variance.axes_manager.set_axis(
-                    invcmaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
-                )
-                self.metadata.Signal.Noise_properties.variance.data = (
-                    self.metadata.Signal.Noise_properties.variance.isig[::-1].data
-                )
-        # create and return new signal
+            s2 = self
         else:
             s2 = self.to_invcm(inplace=inplace, jacobian=jacobian)
-            s2.axes_manager.set_axis(
-                invcmaxis, self.axes_manager.signal_axes[0].index_in_axes_manager
+        # replace axis
+        s2.axes_manager.set_axis(
+            invcmaxis,
+            self.axes_manager.signal_axes[0].index_in_axes_manager,
+        )
+        s2.data = s2.isig[::-1].data
+        # replace variance axis
+        if s2.metadata.has_item("Signal.Noise_properties.variance") and not isinstance(
+            s2.get_noise_variance(), (float, int)
+        ):
+            s2.metadata.Signal.Noise_properties.variance.axes_manager.set_axis(
+                invcmaxis,
+                s2.axes_manager.signal_axes[0].index_in_axes_manager,
             )
-            s2.data = s2.isig[::-1].data
-            # replace variance axis
-            if s2.metadata.has_item(
-                "Signal.Noise_properties.variance"
-            ) and not isinstance(s2.get_noise_variance(), (float, int)):
-                s2.metadata.Signal.Noise_properties.variance.axes_manager.set_axis(
-                    invcmaxis, s2.axes_manager.signal_axes[0].index_in_axes_manager
-                )
-                s2.metadata.Signal.Noise_properties.variance.data = (
-                    s2.metadata.Signal.Noise_properties.variance.isig[::-1].data
-                )
+            s2.metadata.Signal.Noise_properties.variance.data = (
+                s2.metadata.Signal.Noise_properties.variance.isig[::-1].data
+            )
+        if not inplace:
             return s2
 
-    to_invcm_relative.__doc__ %= (TO_INVCM_DOCSTRING, TO_INVCMREL_EXAMPLE)
+    to_invcm_relative.__doc__ %= (
+        TO_INVCM_DOCSTRING,
+        TO_INVCMREL_EXAMPLE,
+    )
 
     # Alias Method Name
     to_raman_shift = to_invcm_relative
@@ -609,7 +461,13 @@ class LumiSpectrum(Signal1D, CommonLumi):
     """
 
     def savetxt(
-        self, filename, fmt="%.5f", delimiter="\t", axes=True, transpose=False, **kwargs
+        self,
+        filename,
+        fmt="%.5f",
+        delimiter="\t",
+        axes=True,
+        transpose=False,
+        **kwargs,
     ):
         """Writes luminescence spectrum object to simple text file.
         %s
@@ -618,7 +476,11 @@ class LumiSpectrum(Signal1D, CommonLumi):
         """
         savetxt(self, filename, fmt, delimiter, axes, transpose, **kwargs)
 
-    savetxt.__doc__ %= (SAVETXT_DOCSTRING, SAVETXT_PARAMETERS, SAVETXT_EXAMPLE)
+    savetxt.__doc__ %= (
+        SAVETXT_DOCSTRING,
+        SAVETXT_PARAMETERS,
+        SAVETXT_EXAMPLE,
+    )
 
     TOARRAY_EXAMPLE = """
     Notes
@@ -660,7 +522,11 @@ class LumiSpectrum(Signal1D, CommonLumi):
         """
         return to_array(self, axes, transpose)
 
-    to_array.__doc__ %= (TOARRAY_DOCSTRING, TOARRAY_PARAMETERS, TOARRAY_EXAMPLE)
+    to_array.__doc__ %= (
+        TOARRAY_DOCSTRING,
+        TOARRAY_PARAMETERS,
+        TOARRAY_EXAMPLE,
+    )
 
     def px_to_nm_grating_solver(
         self,
@@ -711,7 +577,8 @@ class LumiSpectrum(Signal1D, CommonLumi):
             s = self.deepcopy()
 
         s.axes_manager.set_axis(
-            nm_axis, self.axes_manager.signal_axes[0].index_in_axes_manager
+            nm_axis,
+            self.axes_manager.signal_axes[0].index_in_axes_manager,
         )
 
         return s
@@ -720,6 +587,70 @@ class LumiSpectrum(Signal1D, CommonLumi):
         "\n", "\n\t"
     )
 
+    def centroid(self, signal_range=None, **kwargs):
+        """
+        Finds the centroid (center of mass) of a peak in the spectrum from
+        the wavelength (or pixel number) and the intensity at each pixel
+        value. It basically represents a "weighted average" of the peak.
+
+        Notes
+        -----
+        This function only works for a single peak. If you have multiple
+        peaks, slice the signal beforehand or use the signal_range parameter.
+
+        TODO: Implement this function for multiple peaks (with the npeaks
+        parameter) by finding the top 2 peaks from mean spectrum and then
+        returning a signal with 2 com.
+
+        Parameters
+        ----------
+        signal_range : tuple of ints or floats, optional
+            A tuple representing the indices of the signal axis (start index,
+            end index) where the peak is located. If the tuple contains int,
+            it slices on index. If the tuple contains float, it slices on
+            signal units (default HyperSpy s.inav[:] functionality).
+
+        kwargs : dictionary
+            For the scipy.interpolate.interp1d function.
+
+        Returns
+        -------
+        signal : Signal2D, BaseSignal
+            Signal object containing the center of mass for every pixel. Depending
+            on the dimensionality the type is Signal2D or a BaseSignal (for single
+            spectrum).
+        """
+        if signal_range:
+            if type(signal_range) != tuple:
+                raise TypeError(
+                    "The `signal_range` parameter must be a tuple of length 2."
+                )
+            if len(signal_range) != 2:
+                raise ValueError(
+                    f"The `signal_range` parameter must be a tuple of length 2. "
+                    "You passed a tuple of length {len(signal_range)}."
+                )
+
+            s = self.isig[signal_range[0] : signal_range[1]]
+
+        else:
+            s = self
+
+        signal_axis = s.axes_manager.signal_axes[0]
+        center_of_mass = s.map(com, signal_axis=signal_axis, inplace=False)
+
+        # Transfer axes metadata to title
+        center_of_mass.metadata.General.title = f"Centroid map"
+        if signal_axis.name not in (Undefined, ""):
+            center_of_mass.metadata.General.title += f" of {signal_axis.name}"
+        if signal_axis.units not in (Undefined, ""):
+            center_of_mass.metadata.General.title += f" ({signal_axis.units})"
+        if s.metadata.General.title not in (Undefined, ""):
+            center_of_mass.metadata.General.title += f" for {s.metadata.General.title}"
+        if center_of_mass.axes_manager.navigation_size > 0:
+            center_of_mass = center_of_mass.transpose()
+        return center_of_mass
+
 
 class LazyLumiSpectrum(LazySignal, LumiSpectrum):
     """**General lazy 1D luminescence signal class.**"""
