@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2022 The LumiSpy developers
+# Copyright 2019-2023 The LumiSpy developers
 #
 # This file is part of LumiSpy.
 #
@@ -46,8 +46,8 @@ def _n_air(x):
             if wl > 1700:
                 wl = 1700
         else:
-            wl[wl < 185] = 185  # lgtm [py/modification-of-default-value]
-            wl[wl > 1700] = 1700  # lgtm [py/modification-of-default-value]
+            wl[wl < 185] = 185
+            wl[wl > 1700] = 1700
         warn(
             "The wavelength range exceeds the interval of 185 to 1700 nm for "
             "which the exact refractive index of air is used. Beyond this "
@@ -96,7 +96,7 @@ def axis2eV(ax0):
     return axis, factor
 
 
-def data2eV(data, factor, ax0, evaxis):
+def data2eV(data, factor, evaxis, ax0):
     """The intensity is converted from counts/nm (counts/µm) to counts/meV by
     doing a Jacobian transformation, see e.g. Mooney and Kambhampati, J. Phys.
     Chem. Lett. 4, 3316 (2013). Ensures that integrated signals are still
@@ -114,7 +114,7 @@ def data2eV(data, factor, ax0, evaxis):
         return data * factor * c.h * c.c / (c.e * _n_air(ax0.axis[::-1]) * evaxis**2)
 
 
-def var2eV(variance, factor, ax0, evaxis):
+def var2eV(variance, factor, evaxis, ax0):
     """The variance is converted doing a squared Jacobian renormalization to
     match with the transformation of the data.
     """
@@ -160,7 +160,7 @@ def axis2invcm(ax0):
     return axis, factor
 
 
-def data2invcm(data, factor, invcmaxis):
+def data2invcm(data, factor, invcmaxis, ax0=None):
     r"""The intensity is converted from counts/nm (counts/µm) to
     counts/cm$^{-1}$ by doing a Jacobian transformation, see e.g. Mooney and
     Kambhampati, J. Phys. Chem. Lett. 4, 3316 (2013). Ensures that integrated
@@ -169,7 +169,7 @@ def data2invcm(data, factor, invcmaxis):
     return data * factor / (invcmaxis**2)
 
 
-def var2invcm(variance, factor, invcmaxis):
+def var2invcm(variance, factor, invcmaxis, ax0=None):
     r"""The variance is converted doing a squared Jacobian renormalization to
     match with the transformation of the data.
     """
@@ -340,8 +340,6 @@ def join_spectra(S, r=50, scale=True, average=False, kind="slinear"):
             if hasattr(axis, "_expression") or axis.is_uniform:
                 axis.convert_to_non_uniform_axis()
             # 2nd axis does not need to be converted, because it contains axis vector
-            # if hasattr(axis2,'_expression'):
-            #    axis2.convert_to_non_uniform_axis()
             # join axis vectors
             axis.axis = np.hstack((axis.axis[: ind1 + 1], axis2.axis[ind2:]))
             axis.size = axis.axis.size
